/*  ---------------------------------------------------------------------------
    Program  :  mkFontTracer92.sas
    Author   :  Perry Watts

    Project  :  SGF-2011
    Path     :  C:\SGF\SGF11\Fonts4CC\pgm

    Purpose  :  Make a FONT Tracer for the Container lineages in the SAS
                 Styles.Default Template

    Input    :  stylesDefault.txt and tracer.looklike913 (which combines 
                 styles.default and base.template.style templates in 9.2 SAS.
                 See Paper 251-2011 for more information.).

    Output   :  ContainerFontTracer92.HTML
    
    Notes    :  *Program is distributed on an "as is" basis.
    ------------------------------------------------------------------------  */
    %let Datpath=%str(C:\SGF\SGF11\Fonts4CC\dat);
    libname tracer 'C:\SGF\SGF11\Fonts4CC\dat';
    %let odsPath=C:\SGF\SGF11\Fonts4CC\htm;

   data font(keep=start label);
      length start $30 Label $60;
      infile cards length=linelen;
      input Aline $varying80. linelen;
        start=lowCASE(scan(Aline,1,"*"));
        start=compress(start,"'");
        start=left(trim(start));
        label=scan(Aline,2,"*");
        label=compress(label,"=()");
        label=left(trim(label));
/*CARDS: COPIED AND PASTED FROM STYLESDEFAULT92.TXT */
    cards; 
'TitleFont2'* = ("<sans-serif>, Helvetica, sans-serif",4,bold italic)* 
'TitleFont'* = ("<sans-serif>, Helvetica, sans-serif",5,bold italic)*  
'StrongFont'* = ("<sans-serif>, Helvetica, sans-serif",4,bold)*        
'EmphasisFont'* = ("<sans-serif>, Helvetica, sans-serif",3,italic)*    
'FixedEmphasisFont'* = ("<monospace>, Courier, monospace",2,italic)*   
'FixedStrongFont'* = ("<monospace>, Courier, monospace",2,bold)*       
'FixedHeadingFont'* = ("<monospace>, Courier, monospace",2)*           
'BatchFixedFont'* = ("SAS Monospace, <monospace>, Courier, monospace",2)*   
'FixedFont'* = ("<monospace>, Courier",2)*                             
'HeadingEmphasisFont'* = ("<sans-serif>, Helvetica, sans-serif",4,bold italic)*  
'HeadingFont'* = ("<sans-serif>, Helvetica, sans-serif",4,bold)*       
'DocFont'* = ("<sans-serif>, Helvetica, sans-serif",3)*              
   run;
   data cntlinFont(keep=fmtname type start label hlo);
      retain type 'C' fmtname 'FontFm';
      set font end=last;
      output;
      if last then do;
        hlo='O';  label=' '; 
        output;
      end;
    run;
    proc format library=work cntlin=cntlinFont;
    run;

  /* NEED CLASSNUM FOR SORTING PURPOSES */
    data ClassNum(keep=classNum styleElement);
      length styleElement $30;
      retain classNum 0;
      retain styleElement;
      infile "&Datpath.\stylesDefault92.txt" length=linelen;
      input Aline $varying125. linelen;
      Aline=left(aline);
      if index(Aline,"class") eq 1 then do;
        classNum+1;
        styleElement=scan(Aline,2,' ');
      end;
     run;
     
  /* CONVENIENTLY SUBSET TRACER.LOOKLIKE913 */   
     data FontContainer1(keep=StyleName FontName);
       set tracer.looklike913;
       if attributeName eq 'FONT';
       fontName=lowcase(thisFontName);
     run;

     proc sql noprint;
       create table fontContainer as
       select classNum, StyleName, FontName
       from classNum, fontContainer1
       where upcase(styleElement) eq upcase(StyleName);
    quit;

  /* CLASSNUM RETAINS THE CLASS ORDER IN STYLES.DEFAULT. FOR EXAMPLE,
     CONTAINER WILL BE FIRST IN THE LIST WHEN FONTNAME=DOCFONT .*/
    proc sort data=fontContainer nodup;
      by fontname classNum;
    run;
 
  /* THIS IS THE DATA SET THAT IS USED AS INPUT TO ODS-HTML. */
   data fontTracer(keep=font fontName styleName);
     label font='Font' fontName='Font Name' styleName='Style Element';
     length font $60;
     set fontContainer;
     font=put(fontName,$fontFm.);
     font=compress(font,"()");
    run;
  
   proc template;
    define style styles.fontStyle;
    parent=styles.default;
    style output / 
         background = cxD9D9D9
         rules = ALL
         frame = BOX
         cellpadding = 3
         cellspacing = 2
         bordercolor = cx888888
         borderwidth = 3;
    style data/
         foreground=black
         background=CXF8F8F8;
    style header / 
         background=gray
         foreground=white
         font = (Arial,12pt,Bold);
    style SystemTitle / 
         foreground=black
         font=("Arial",15pt,Bold);
    end; 
   run;
 
  proc sql noprint;
    create table uniqueFonts as 
      select distinct font from fontTracer;
    select count(*) into :n
      from uniqueFonts;
    select font into :f1 - :f%left(&n)
      from uniqueFonts;
  quit;
  %let n = %left(%trim(&n));

  options mprint;
  %macro tableTemplate;
  proc template;
     define Table TABLES.fontTbl;
       dynamic colhd;
       mvar %do i = 1 %to &n; f&i %end; ;
       classlevels=on;
       define column columnFont;
         blank_dups=on;
         define header colhd;
           style=header;
           just=Center;
         end;
         header=colhd;
        cellstyle 
           %do i= 1 %to &n;
             _val_ EQ F&i as {font = (&&F&i)},
           %end;
           1 as data;
       end;  /* DEFINE COLUMN */
       define column restOfData;
         generic=on;
         blank_dups=on;
         define header colhd;
           style=header;
           just=Center;
         end;
         header=colhd;
       end; /* DEFINE COLUMN */
     end;   /* DEFINE TABLE */
   run;
   %mend tableTemplate;
   %tableTemplate
  
   ods listing close;
   ods HTML
      style= styles.fontStyle
      path = "&odsPath" (URL=NONE)
      body = 'containerFontTracer92.HTML'; 
   title1 "Font Tracer for the SAS STYLES.DEFAULT Container Style Elements";

     data _null_;
      set fontTracer;
       file print ods=(template='TABLES.fontTbl'
                      columns=(
                      columnfont=font(dynamic=(colhd='Font'))
                      restOfData=fontName(generic=on dynamic=(colhd='Font Name'))
                      restOfData=styleName(generic=on dynamic=(colhd='Style Element'))
                              )
                      );
      put _ods_;
     run;
   ods _all_ close;
   ods listing;
